﻿// #######################################################################################################
// Program Name: CCR Server.exe
// Author: Brice Hilgemann
// Assistance: Professor Blahnik with TcpClient information, MSDN documentation on sockets
// Last Modified: 2/13/2010
// Status: In Progress
// Usage: Creates a Server that has the ability to have multiple clients attached to it in which communcation
//        can occur asynchronously with these and a server can sends information to all of them.
// #######################################################################################################

using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;



// #####################################
//      Picturebox Descriptions:
// #####################################
//
//   box1 - banner
//   box2 - layout
//
// -------------------------------------
//   Turnouts (Odd numbers below pair)
// -------------------------------------
//   box3-4   : turnouts upper far left
//   box5-6   : turnouts upper left
//   box7-8   : turnouts upper right
//   box9-10  : turnouts upper far right
//   box11-12 : turnouts bottom right
//   box13-14 : turnouts bottom left 
//
// -------------------------------------
//   Sensors
// -------------------------------------
//   box15-29 : top sensors
//   box30-35 : curve sensors
//   box37-43 : bottom sensors
//
// -------------------------------------
//   Track
// -------------------------------------
//   box44-51 : top track
//   box52-58 : top lower track
//   box59-67 : bottom track
//   box68-69 : left curve
//   box70-71 : right curve
//
// -------------------------------------
//   'Buttons'
// -------------------------------------
//   box72    : add train
//   box76    : remove train


// #####################################
//      ErrorProvider Descriptions:
// #####################################
//
//   errorProvider1 : textBox2
//   errorProvider2 : listBox2
//   errorProvider3 : listBox3
//


// #####################################
//      label Descriptions (visible):
// #####################################
//
//   label1     : train count label
//   label2     : connection count label
//   label5     : ip and port label at top
//   label6-11  : turnout labels
//   label19-22 : add train dialog labels
//   label23-53 : track labels
//   label54    : available track label
//

namespace Server
{
    public partial class Form1 : Form
    {
        Socket ServerSocket;                                     // Socket for the Server
        Socket[] Clients = new Socket[100];                      // Array of ClientStates for multiple clients
        AsyncCallback CallBack;

        byte[] dataholder = new byte[512];                       // Will act as a buffer for our information
        int connection_count = 0;                                // Number of client connections
        int total_connections = 0;                               // Counts total connections made throughout run-time
        int total_trains = 0;
        int a = 0;    //test

        public Form1(string com_selected)
        {
            InitializeComponent();

            MessageBox.Show("COM selected is " + com_selected);
        }

        public void Form1_Shown(object sender, EventArgs e)
        // ################################################################################################################
        // Set up defaults at startup when form is first shown
        // ################################################################################################################
        {
            string strHostName = System.Net.Dns.GetHostName();
            IPHostEntry ipEntry = System.Net.Dns.GetHostEntry(strHostName);     // Get current IP Address of system
            IPAddress[] addr = ipEntry.AddressList;

            label5.Text = addr[addr.Length - 1].ToString() + " at Port 10000";
            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server started with IP " + addr[addr.Length - 1].ToString() + ":10000");

            Form1.ActiveForm.FormBorderStyle = FormBorderStyle.FixedSingle;
            Form1.ActiveForm.StartPosition = FormStartPosition.CenterScreen;    // Default form settings
            Form1.ActiveForm.MaximizeBox = false;

            SetupListViews();                          // Creates listview columns
            SetupColors();                             // Sets up color selection based off of file
            SetupTrackParts();                         // Sets up track parts based off of file
            SetupTurnouts();                           // Sets up turnouts based off of file
            SetLabelAttributes();                      // Sets up label attributes
            DefaultImages();                           // Hides and displays proper images

            textBox1.Enabled = false;
            button1.Enabled = false;

            try
            {
                ServerSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);         // Acts as listener
                IPEndPoint chosen_ip = new IPEndPoint(IPAddress.Any, 10000);                                        // Creates server at default address with port 10000
                ServerSocket.Bind(chosen_ip);                                                                       // Bind the IP Address to listener
                ServerSocket.Listen(5);                                                                             // Create listening queue with queue backlog of 5

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                             // Set callback to HandleClientConnect if someone tries to connect
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue " + ex);         // If something with socket fails
            }

            timer1.Interval = 500;
            //timer1.Enabled = true;
        }

        public void SetupListViews()
        {
            listView1.GridLines = true;
            listView1.View = View.Details;
            listView1.Columns.Add("Train ID", 50, HorizontalAlignment.Center);           //Setup listview
            listView1.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView1.Columns.Add("Count", 40, HorizontalAlignment.Center);
            listView1.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView1.Columns.Add("IP and Port", 125, HorizontalAlignment.Center);
            listView1.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView2.GridLines = true;
            listView2.View = View.Details;
            listView2.Columns.Add("ID", 30, HorizontalAlignment.Center);           //Setup listview
            listView2.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView2.Columns.Add("State", 50, HorizontalAlignment.Center);
            listView2.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView2.Columns.Add("Owner", 50, HorizontalAlignment.Center);
            listView2.Columns[2].TextAlign = HorizontalAlignment.Center;

            listView3.GridLines = true;
            listView3.View = View.Details;
            listView3.Columns.Add("ID", 30, HorizontalAlignment.Center);           //Setup listview
            listView3.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView3.Columns.Add("State", 50, HorizontalAlignment.Center);
            listView3.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView4.GridLines = true;
            listView4.View = View.Details;
            listView4.Columns.Add("Left", 40, HorizontalAlignment.Center);           //Setup listview
            listView4.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Left2", 40, HorizontalAlignment.Center);
            listView4.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right", 40, HorizontalAlignment.Center);
            listView4.Columns[2].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right2", 45, HorizontalAlignment.Center);
            listView4.Columns[3].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Turn ID", 50, HorizontalAlignment.Center);
            listView4.Columns[4].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Speed Lmt", 65, HorizontalAlignment.Center);
            listView4.Columns[5].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Trk Lgth", 60, HorizontalAlignment.Center);
            listView4.Columns[6].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Turn St", 50, HorizontalAlignment.Center);
            listView4.Columns[7].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Left Sens", 60, HorizontalAlignment.Center);
            listView4.Columns[8].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right Sens", 65, HorizontalAlignment.Center);
            listView4.Columns[9].TextAlign = HorizontalAlignment.Center;

            listView5.GridLines = true;
            listView5.View = View.Details;
            listView5.Columns.Add("Train ID", 60, HorizontalAlignment.Center);       //Setup listview
            listView5.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView5.Columns.Add("Turn ID", 60, HorizontalAlignment.Center);
            listView5.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView5.Columns.Add("Turn St", 60, HorizontalAlignment.Center);
            listView5.Columns[2].TextAlign = HorizontalAlignment.Center;

            listView6.GridLines = true;
            listView6.View = View.Details;
            listView6.Columns.Add("Train ID", 55, HorizontalAlignment.Center);           //Setup listview
            listView6.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView6.Columns.Add("Track ID", 60, HorizontalAlignment.Center);
            listView6.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView7.GridLines = true;
            listView7.View = View.Details;
            listView7.Columns.Add("Train ID", 60, HorizontalAlignment.Center);           //Setup listview
            listView7.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Owner", 45, HorizontalAlignment.Center);
            listView7.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Track ID", 60, HorizontalAlignment.Center);
            listView7.Columns[2].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Request", 60, HorizontalAlignment.Center);
            listView7.Columns[3].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Blocked", 60, HorizontalAlignment.Center);
            listView7.Columns[4].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Speed", 45, HorizontalAlignment.Center);
            listView7.Columns[5].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Direct.", 60, HorizontalAlignment.Center);
            listView7.Columns[6].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Orient.", 60, HorizontalAlignment.Center);
            listView7.Columns[7].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Light", 45, HorizontalAlignment.Center);
            listView7.Columns[8].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Horn", 40, HorizontalAlignment.Center);
            listView7.Columns[9].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Next Sens", 65, HorizontalAlignment.Center);
            listView7.Columns[10].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Speed Lmt", 65, HorizontalAlignment.Center);
            listView7.Columns[11].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Color", 40, HorizontalAlignment.Center);
            listView7.Columns[11].TextAlign = HorizontalAlignment.Center;
        }

        public void SetupColors()
        {
            StreamReader sr;

            if (!File.Exists("colors.txt"))
            //Open up default track layout for the server to get the track parts and default settings a little more dynamically
            {
                MessageBox.Show("Color file does not exist - starting standard");

                comboBox3.Items.Add("blue");
                comboBox3.Items.Add("red");

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default colors set from hard-coded");
            }
            else
            {
                sr = new StreamReader("colors.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    comboBox3.Items.Add(line);
                }

                sr.Close();

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default colors set from file");
            }

            comboBox3.SelectedIndex = 0;
        }

        public void SetupTrackParts()
        {
            StreamReader sr;
            ListViewItem lvi;
            ListViewItem.ListViewSubItem lviSub1;
            int counter = 0;

            if (!File.Exists("track.txt"))
            //Open up default track layout for the server to get the track parts and default settings a little more dynamically
            {
                MessageBox.Show("Track file does not exist - Closing program as this is crucial.  Review help to set this up");

                Form1_FormClosing(null, null);
            }
            else
            {
                sr = new StreamReader("track.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    string[] dd = line.Split(';');

                    lvi = new ListViewItem(dd[0].ToString());                              // Add left pointer

                    for (int i = 1; i < 10; i++)
                    {
                        lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[i].ToString());     // Add left pointer2
                        lvi.SubItems.Add(lviSub1);
                    }

                    listView4.Items.Add(lvi);

                    lvi = new ListViewItem("");

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[4].ToString());     // Add turnout id
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[7].ToString());     // Add turnout state
                    lvi.SubItems.Add(lviSub1);

                    listView5.Items.Add(lvi);

                    listBox2.Items.Add(counter.ToString() + " - Available");
                    
                    counter++;
                }

                listBox2.SelectedIndex = 0;              //Select the first index
                sr.Close();                              //Close file reader

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default track set from file");
            }
        }

        public void SetupTurnouts()
        {
            StreamReader sr;
            ListViewItem lvi;
            ListViewItem.ListViewSubItem lviSub1;  
  
            if (!File.Exists("turnouts.txt"))
            //Open up default turnout layout for the server to get the turnout names and default settings a little more dynamically
            {
                MessageBox.Show("Turnouts file does not exist - starting standard");
                int offset = 4;

                for (int i = 0; i < 6; i++)
                {
                    lvi = new ListViewItem(offset.ToString());

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, "top");     // Add to listview
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, "none");    // Add to listview
                    lvi.SubItems.Add(lviSub1);

                    listView2.Items.Add(lvi);
                    offset = (offset * 2);
                }

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default turnouts set from program hard-coded");
            }
            else
            {
                sr = new StreamReader("turnouts.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    string[] dd = line.Split(';');

                    lvi = new ListViewItem(dd[0].ToString());

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[1].ToString());     // Add to listview
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[2].ToString());     // Add to listview
                    lvi.SubItems.Add(lviSub1);

                    listView2.Items.Add(lvi);
                }

                sr.Close();                              //Close file reader
                
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default turnouts set from file");
            }
        }

        public void SetLabelAttributes()
        {
            label6.BackColor = Color.FromArgb(117, 131, 58);
            label7.BackColor = Color.FromArgb(117, 131, 58);
            label8.BackColor = Color.FromArgb(117, 131, 58);
            label9.BackColor = Color.FromArgb(117, 131, 58);
            label10.BackColor = Color.FromArgb(117, 131, 58);
            label11.BackColor = Color.FromArgb(117, 131, 58);

            label6.ForeColor = Color.FromArgb(243, 237, 156);
            label7.ForeColor = Color.FromArgb(243, 237, 156);
            label8.ForeColor = Color.FromArgb(243, 237, 156);
            label9.ForeColor = Color.FromArgb(243, 237, 156);
            label10.ForeColor = Color.FromArgb(243, 237, 156);
            label11.ForeColor = Color.FromArgb(243, 237, 156);

         //Goes from top row left-to-right and then bottom row left-to-right
            label6.Text = listView2.Items[0].SubItems[0].Text;
            label7.Text = listView2.Items[1].SubItems[0].Text;
            label8.Text = listView2.Items[2].SubItems[0].Text;
            label9.Text = listView2.Items[3].SubItems[0].Text;
            label11.Text = listView2.Items[4].SubItems[0].Text;
            label10.Text = listView2.Items[5].SubItems[0].Text;

         //Set up the combox attributes
            comboBox1.SelectedIndex = 0;
            comboBox2.SelectedIndex = 0;

         //Set up label colors for track pieces - (labels 23-53)
            label23.ForeColor = Color.FromArgb(180, 210, 40);
            label23.BackColor = Color.FromArgb(117, 131, 58);
            label24.ForeColor = Color.FromArgb(180, 210, 40);
            label24.BackColor = Color.FromArgb(117, 131, 58);
            label25.ForeColor = Color.FromArgb(180, 210, 40);
            label25.BackColor = Color.FromArgb(117, 131, 58);
            label26.ForeColor = Color.FromArgb(180, 210, 40);
            label26.BackColor = Color.FromArgb(117, 131, 58);
            label27.ForeColor = Color.FromArgb(180, 210, 40);
            label27.BackColor = Color.FromArgb(117, 131, 58);
            label28.ForeColor = Color.FromArgb(180, 210, 40);
            label28.BackColor = Color.FromArgb(117, 131, 58);
            label29.ForeColor = Color.FromArgb(180, 210, 40);
            label29.BackColor = Color.FromArgb(117, 131, 58);
            label30.ForeColor = Color.FromArgb(180, 210, 40);
            label30.BackColor = Color.FromArgb(117, 131, 58);
            label31.ForeColor = Color.FromArgb(180, 210, 40);
            label31.BackColor = Color.FromArgb(117, 131, 58);
            label32.ForeColor = Color.FromArgb(180, 210, 40);
            label32.BackColor = Color.FromArgb(117, 131, 58);
            label33.ForeColor = Color.FromArgb(180, 210, 40);
            label33.BackColor = Color.FromArgb(117, 131, 58);
            label34.ForeColor = Color.FromArgb(180, 210, 40);
            label34.BackColor = Color.FromArgb(117, 131, 58);
            label35.ForeColor = Color.FromArgb(180, 210, 40);
            label35.BackColor = Color.FromArgb(117, 131, 58);
            label36.ForeColor = Color.FromArgb(180, 210, 40);
            label36.BackColor = Color.FromArgb(117, 131, 58);
            label37.ForeColor = Color.FromArgb(180, 210, 40);
            label37.BackColor = Color.FromArgb(117, 131, 58);
            label38.ForeColor = Color.FromArgb(180, 210, 40);
            label38.BackColor = Color.FromArgb(117, 131, 58);
            label39.ForeColor = Color.FromArgb(180, 210, 40);
            label39.BackColor = Color.FromArgb(117, 131, 58);
            label40.ForeColor = Color.FromArgb(180, 210, 40);
            label40.BackColor = Color.FromArgb(117, 131, 58);
            label41.ForeColor = Color.FromArgb(180, 210, 40);
            label41.BackColor = Color.FromArgb(117, 131, 58);
            label42.ForeColor = Color.FromArgb(180, 210, 40);
            label42.BackColor = Color.FromArgb(117, 131, 58);
            label43.ForeColor = Color.FromArgb(180, 210, 40);
            label43.BackColor = Color.FromArgb(117, 131, 58);
            label44.ForeColor = Color.FromArgb(180, 210, 40);
            label44.BackColor = Color.FromArgb(117, 131, 58);
            label45.ForeColor = Color.FromArgb(180, 210, 40);
            label45.BackColor = Color.FromArgb(117, 131, 58);
            label46.ForeColor = Color.FromArgb(180, 210, 40);
            label46.BackColor = Color.FromArgb(117, 131, 58);
            label47.ForeColor = Color.FromArgb(180, 210, 40);
            label47.BackColor = Color.FromArgb(117, 131, 58);
            label48.ForeColor = Color.FromArgb(180, 210, 40);
            label48.BackColor = Color.FromArgb(117, 131, 58);
            label49.ForeColor = Color.FromArgb(180, 210, 40);
            label49.BackColor = Color.FromArgb(117, 131, 58);
            label50.ForeColor = Color.FromArgb(180, 210, 40);
            label50.BackColor = Color.FromArgb(117, 131, 58);
            label51.ForeColor = Color.FromArgb(180, 210, 40);
            label51.BackColor = Color.FromArgb(117, 131, 58);
            label52.ForeColor = Color.FromArgb(180, 210, 40);
            label52.BackColor = Color.FromArgb(117, 131, 58);
            label53.ForeColor = Color.FromArgb(180, 210, 40);
            label53.BackColor = Color.FromArgb(117, 131, 58);
        }

        public void DefaultImages()
        {
            //Top sensors
            pictureBox15.Visible = false;
            pictureBox16.Visible = false;
            pictureBox17.Visible = false;
            pictureBox18.Visible = false;
            pictureBox19.Visible = false;
            pictureBox20.Visible = false;
            pictureBox21.Visible = false;
            pictureBox22.Visible = false;
            pictureBox23.Visible = false;
            pictureBox24.Visible = false;
            pictureBox25.Visible = false;
            pictureBox26.Visible = false;
            pictureBox27.Visible = false;
            pictureBox28.Visible = false;
            pictureBox29.Visible = false;

            //Curve sensors
            pictureBox30.Visible = false;
            pictureBox31.Visible = false;
            pictureBox32.Visible = false;
            pictureBox33.Visible = false;
            pictureBox34.Visible = false;
            pictureBox35.Visible = false;

            //Bottom sensors
            pictureBox36.Visible = false;
            pictureBox37.Visible = false;
            pictureBox38.Visible = false;
            pictureBox39.Visible = false;
            pictureBox40.Visible = false;
            pictureBox41.Visible = false;
            pictureBox42.Visible = false;
            pictureBox43.Visible = false;

            pictureBox44.Visible = false;
            pictureBox45.Visible = false;
            pictureBox46.Visible = false;
            pictureBox47.Visible = false;
            pictureBox48.Visible = false;
            pictureBox49.Visible = false;
            pictureBox50.Visible = false;
            pictureBox51.Visible = false;
            pictureBox52.Visible = false;
            pictureBox53.Visible = false;
            pictureBox54.Visible = false;
            pictureBox55.Visible = false;
            pictureBox56.Visible = false;
            pictureBox57.Visible = false;
            pictureBox58.Visible = false;
            pictureBox59.Visible = false;
            pictureBox60.Visible = false;
            pictureBox61.Visible = false;
            pictureBox62.Visible = false;
            pictureBox63.Visible = false;
            pictureBox64.Visible = false;
            pictureBox65.Visible = false;
            pictureBox66.Visible = false;
            pictureBox67.Visible = false;
            pictureBox68.Visible = false;
            pictureBox69.Visible = false;
            pictureBox70.Visible = false;
            pictureBox71.Visible = false;

            //Show default turnout images
            ShowTurnoutImages(0, pictureBox3, pictureBox4);
            ShowTurnoutImages(1, pictureBox5, pictureBox6);
            ShowTurnoutImages(2, pictureBox7, pictureBox8);
            ShowTurnoutImages(3, pictureBox9, pictureBox10);
            ShowTurnoutImages(4, pictureBox13, pictureBox14);
            ShowTurnoutImages(5, pictureBox11, pictureBox12);
        }

        public void ShowTurnoutImages(int turnout_id, PictureBox bottom, PictureBox top)
        {
            if (listView2.Items[turnout_id].SubItems[1].Text == "top")
            {
                top.Image = Properties.Resources.red_turnout;
                bottom.Image = Properties.Resources.off_turnout;
            }
            else
            {
                top.Image = Properties.Resources.off_turnout;
                bottom.Image = Properties.Resources.green_turnout;
            }
        }

        public void HandleClientConnect(IAsyncResult sync_result)
        // ################################################################################################################
        // Connection to client initiated
        //   - New connection attempt has been made, process
        // ################################################################################################################
        {
            try
            {
                Clients[total_connections] = (Socket)ServerSocket.EndAccept(sync_result);               // Process EndAccept and save socket information

                ListViewItem lvi;
                ListViewItem.ListViewSubItem lviSub1;                                                   // Create listview items

                lvi = new ListViewItem("");
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, total_connections.ToString());     // Add to listview
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, Clients[total_connections].RemoteEndPoint.ToString());     // Add to listview
                lvi.SubItems.Add(lviSub1);

                listView1.Items.Add(lvi);

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server connected to " + Clients[total_connections].RemoteEndPoint.ToString());

                HandleDatafromClient(Clients[total_connections]);                                       // Call HandleDatafromClient to set up Callback

                total_connections++;                                                                    // Increase connections
                connection_count++;

                textBox1.Enabled = true;
                button1.Enabled = true;

                label2.Text = "Connections: " + connection_count;

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                 // Start another callback to listen for more connections
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue2 " + ex);         // If something with socket fails
            }
        }

        public void HandleDatafromClient(Socket HolderSocket)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
        {
            try
            {
                if (CallBack == null)
                {
                    CallBack = new AsyncCallback(HandleDataReceivedfromClient);                         // Set it up to create a callback to HandleDataReceivedfromClient
                }

                HolderSocket.BeginReceive(dataholder, 0, dataholder.Length, SocketFlags.None, CallBack, HolderSocket); // BeginReceive on socket
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue3 " + ex);           // If something with socket fails
            }
        }

        public void HandleDataReceivedfromClient(IAsyncResult sync_result)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
		{
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;                    // Get information sent in temporary socket

                int received = TempSocket.EndReceive(sync_result);              // Store it in an int to then decode

                if (received > 0)
                {
                    string received_info = Encoding.ASCII.GetString(dataholder, 0, received);    // Convert received to string

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - From Client " + TempSocket.RemoteEndPoint.ToString() + " " + received_info);    //Output string received



                    // ****************************************************************************************************
                    //
                    //
                    //           PROCESS REQUEST FROM CLIENT AND SEND A RESPONSE OF SOMETHING TO THEM
                    //
                    //
                    // ****************************************************************************************************

                    if (received_info == "Increase a")
                    {
                        a++;
                    }
                    else
                    {
                        a--;
                    }

                    string server_response = "Server Response";
                    byte[] response = Encoding.ASCII.GetBytes(server_response);
                    TempSocket.BeginSend(response, 0, response.Length, SocketFlags.None, new AsyncCallback(ServerSendResponse), TempSocket);

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent response of " + server_response + " to Client " + TempSocket.RemoteEndPoint.ToString());
                }
                else
                {
                    CloseSocket(TempSocket);
                }
            }
            catch (ObjectDisposedException)
            {
                CloseSocket(TempSocket);
            }
            catch (SocketException)
            {
                CloseSocket(TempSocket);
            }
            catch (Exception)
            {
                CloseSocket(TempSocket);
            }
		}

        public void ServerSendResponse(IAsyncResult sync_result)
        {
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;

                int received = TempSocket.EndReceive(sync_result);
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue4 " + ex);           // If something with socket fails
                CloseSocket(TempSocket);
            }

            HandleDatafromClient(TempSocket);
        }


        private void CloseSocket(Socket TempSocket)
        {
            for (int i = 0; i < connection_count; i++)
            {
                if (TempSocket.RemoteEndPoint.ToString() == listView1.Items[i].SubItems[2].Text)
                {
                    listView1.Items[i].Remove();

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Client " + TempSocket.RemoteEndPoint.ToString() + " disconnected");

                    TempSocket.Close();
                    TempSocket = null;

                    connection_count--;

                    if (connection_count == 0)
                    {
                        textBox1.Enabled = false;
                        button1.Enabled = false;
                    }

                    label2.Text = "Connections: " + connection_count;

                    return;
                }
            }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (ServerSocket != null)
            {
                ServerSocket.Close();
            }
            for(int i=0; i < total_connections; i++)     //Go through connection count, close client sockets
            {
				Clients[i].Close();                     //Close and null it
			}

            Application.Exit();
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            try
            {
                byte[] data = Encoding.ASCII.GetBytes(textBox1.Text);

                for (int i = 0; i < connection_count; i++)
                {
                    try
                    {
                        Clients[Convert.ToInt32(listView1.Items[i].SubItems[1].Text)].Send(data);
                    }
                    catch             //Client is no longer there, move up other clients
                    {
                        MessageBox.Show("Client " + listView1.Items[i].SubItems[1].Text + " disconnected?");
                    }
                }
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue5 " + ex);           // If something with socket fails
            }

            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent " + textBox1.Text + " to Clients");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            label4.Text = a.ToString();                                         // Update this
        }

        //######################################################################
        //        TURNOUT EVENTS
        //######################################################################

        private void pictureBox4_Click(object sender, EventArgs e)
        {
            CheckTurnout(0, "top", pictureBox3, pictureBox4);
        }
        private void pictureBox3_Click(object sender, EventArgs e)
        {
            CheckTurnout(0, "bottom", pictureBox3, pictureBox4);
        }


        private void pictureBox6_Click(object sender, EventArgs e)
        {
            CheckTurnout(1, "top", pictureBox5, pictureBox6);
        }
        private void pictureBox5_Click(object sender, EventArgs e)
        {
            CheckTurnout(1, "bottom", pictureBox5, pictureBox6);
        }


        private void pictureBox8_Click(object sender, EventArgs e)
        {
            CheckTurnout(2, "top", pictureBox7, pictureBox8);
        }
        private void pictureBox7_Click(object sender, EventArgs e)
        {
            CheckTurnout(2, "bottom", pictureBox7, pictureBox8);
        }


        private void pictureBox10_Click(object sender, EventArgs e)
        {
            CheckTurnout(3, "top", pictureBox9, pictureBox10);
        }
        private void pictureBox9_Click(object sender, EventArgs e)
        {
            CheckTurnout(3, "bottom", pictureBox9, pictureBox10);
        }


        private void pictureBox14_Click(object sender, EventArgs e)
        {
            CheckTurnout(4, "top", pictureBox13, pictureBox14);
        }
        private void pictureBox13_Click(object sender, EventArgs e)
        {
            CheckTurnout(4, "bottom", pictureBox13, pictureBox14);
        }
        
        
        private void pictureBox12_Click(object sender, EventArgs e)
        {
            CheckTurnout(5, "top", pictureBox11, pictureBox12);
        }
        private void pictureBox11_Click(object sender, EventArgs e)
        {
            CheckTurnout(5, "bottom", pictureBox11, pictureBox12);
        }


        public void CheckTurnout(int turnout_id, string location, PictureBox bottom, PictureBox top)
        {
            //Enter monitor
            Monitor.Enter(listView2);

            if(listView2.Items[turnout_id].SubItems[2].Text == "none" && listView2.Items[turnout_id].SubItems[1].Text != location)
            {
                listView2.Items[turnout_id].SubItems[1].Text = location;

                ShowTurnoutImages(turnout_id, bottom, top);

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Turnout " + listView2.Items[turnout_id].SubItems[0].Text + " changed to " + listView2.Items[turnout_id].SubItems[1].Text);
            }

            Monitor.Exit(listView2);            
            //Exit monitor
        }
    }
}
