﻿// #######################################################################################################
// Program Name: CCR Server.exe
// Author: Brice Hilgemann
// Assistance: Professor Blahnik with TcpClient information, MSDN documentation on sockets
// Last Modified: 2/13/2010
// Status: In Progress
// Usage: Creates a Server that has the ability to have multiple clients attached to it in which communcation
//        can occur asynchronously with these and a server can sends information to all of them.
// #######################################################################################################

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.IO.Pipes;                  //needed for named pipes
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Server
{
    public partial class Form1 : Form
    {
        /*[DllImport("tos_api.dll")]  //Initializes DCC for sending/receiving using tos_api.dll
        public static extern bool TOS_InitDCC(string port, IntPtr pipe);

        [DllImport("kmp_client_api.dll", EntryPoint = "SendMessageToPeer")]//Sends messages to thread clients
        public static extern bool SendMessageToPeer(string message, UInt16 length, UInt16 code, string Destination, IntPtr pipe);
        //SendMessageToPeer(char * MessageData, unsigned long Length, unsigned long ServiceCode, char * Destination, HANDLE pipe)

        [DllImport("tos_api.dll")]//Reads the AIU's on the track for the track status
        public static extern bool TOS_ReadAIU(uint address, uint input, IntPtr pipe);
        //bool TOS_ReadAIU(unsigned long Address, unsigned long Input, HANDLE pipe);

        [DllImport("tos_api.dll")]//Modifies train engine speed and direction
        public static extern bool TOS_SetTrainSpeedDCC(uint TrainAddress, uint Speed, byte direction, IntPtr pipe);
        //TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe)

        [DllImport("tos_api.dll")]//Sets turnout left or right at the address specified
        public static extern bool TOS_SetTurnout(uint Address, byte value, IntPtr pipe);
        //TOS_SetTurnout(unsigned long Address, unsigned char value, HANDLE pipe)

        [DllImport("kmp_client_api.dll")]//Gets the message waiting on the TOS_UI pipe (from dccman)
        public static extern int GetMessageFromPeer(ref byte From, ref uint ServiceCode, ref byte MessageData, ref uint MessageLength, IntPtr pipe);
        //int GetMessageFromPeer(char * From, unsigned long& ServiceCode, char * MessageData, unsigned long& MessageLength, HANDLE pipe)
        */

        public delegate void WritetoLogsCallback(string loginfo);
        public delegate void UpdateLabelsCallback(Label label, string item);
        public delegate void CommandRailroadCallback(string command, string[] data);

        ProcessStartInfo test = new ProcessStartInfo();          // Process that creates all the pipes
        ProcessStartInfo DCCMan = new ProcessStartInfo();        // Starts the DCCMan worker thread for track control
        Process exe1;                                            // This will become test.exe
        Process exe2;                                            // This will become DCCMan.exe

        NamedPipeClientStream pipe_to_railroad = new NamedPipeClientStream(".", "TOSUI", PipeDirection.InOut);

        Socket ServerSocket;                                     // Socket for the Server
        Socket[] Clients = new Socket[100];                      // Array of ClientStates for multiple clients
        AsyncCallback CallBack;

        byte[] dataholder = new byte[512];                       // Will act as a buffer for our information
        int connection_count = 0;                                // Number of client connections
        int total_connections = 0;                               // Counts total connections made throughout run-time
        int total_trains = 0;
        int sensor_count = 29;
        int track_count = 31;
        PictureBox[] track = new PictureBox[31];
        PictureBox[] sensors = new PictureBox[29];
        int a = 0;    //test
        SerialPort port = new SerialPort();
        string com_port;

        public Form1(string com_selected)
        {
            InitializeComponent();

            com_port = com_selected;
        }

        public void Form1_Shown(object sender, EventArgs e)
        // ################################################################################################################
        // Set up defaults at startup when form is first shown
        // ################################################################################################################
        {
            string strHostName = System.Net.Dns.GetHostName();
            IPHostEntry ipEntry = System.Net.Dns.GetHostEntry(strHostName);     // Get current IP Address of system
            IPAddress[] addr = ipEntry.AddressList;

            label5.Text = addr[addr.Length - 1].ToString() + " at Port 10000";
            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server started with IP " + addr[addr.Length - 1].ToString() + ":10000");

            Form1.ActiveForm.FormBorderStyle = FormBorderStyle.FixedSingle;
            Form1.ActiveForm.StartPosition = FormStartPosition.CenterScreen;    // Default form settings
            Form1.ActiveForm.MaximizeBox = false;

            initDCC(com_port);

            SetupListViews();                          // Creates listview columns
            SetupColors();                             // Sets up color selection based off of file
            SetupTrackParts();                         // Sets up track parts based off of file
            SetupTurnouts();                           // Sets up turnouts based off of file
            SetLabelAttributes();                      // Sets up label attributes
            DefaultImages();                           // Hides and displays proper images

            textBox1.Enabled = false;
            button1.Enabled = false;

            try
            {
                ServerSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);         // Acts as listener
                IPEndPoint chosen_ip = new IPEndPoint(IPAddress.Any, 10000);                                        // Creates server at default address with port 10000
                ServerSocket.Bind(chosen_ip);                                                                       // Bind the IP Address to listener
                ServerSocket.Listen(5);                                                                             // Create listening queue with queue backlog of 5

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                             // Set callback to HandleClientConnect if someone tries to connect
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue " + ex);         // If something with socket fails
            }

            timer1.Interval = 1000;
            timer1.Enabled = true;
        }

        private void initDCC(string com_port)
        {
            port = new SerialPort(com_port, 9600, Parity.None, 8, StopBits.One);

            try
            {
                port.BaudRate = 9600;
                port.DataBits = 8;
                port.Parity = Parity.None;
                port.StopBits = StopBits.One;
                port.DtrEnable = true;
                port.Handshake = Handshake.None;
                port.DtrEnable = true;
                port.Open();
            }
            catch
            {
                MessageBox.Show("Issue opening up the COM Port");
            }
        }

        public void SetupListViews()
        {
            listView1.GridLines = true;
            listView1.View = View.Details;
            listView1.Sorting = SortOrder.None;
            listView1.Columns.Add("Train ID", 50, HorizontalAlignment.Center);           //Setup listview
            listView1.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView1.Columns.Add("Count", 40, HorizontalAlignment.Center);
            listView1.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView1.Columns.Add("IP and Port", 125, HorizontalAlignment.Center);
            listView1.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView2.GridLines = true;
            listView2.View = View.Details;
            listView2.Sorting = SortOrder.None;
            listView2.Columns.Add("ID", 30, HorizontalAlignment.Center);           //Setup listview
            listView2.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView2.Columns.Add("State", 50, HorizontalAlignment.Center);
            listView2.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView2.Columns.Add("Owner", 50, HorizontalAlignment.Center);
            listView2.Columns[2].TextAlign = HorizontalAlignment.Center;

            listView3.GridLines = true;
            listView3.View = View.Details;
            listView3.Sorting = SortOrder.None;
            listView3.Columns.Add("ID", 30, HorizontalAlignment.Center);           //Setup listview
            listView3.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView3.Columns.Add("State", 50, HorizontalAlignment.Center);
            listView3.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView4.GridLines = true;
            listView4.View = View.Details;
            listView4.Sorting = SortOrder.None;
            listView4.Columns.Add("Left", 40, HorizontalAlignment.Center);           //Setup listview
            listView4.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Left2", 40, HorizontalAlignment.Center);
            listView4.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right", 40, HorizontalAlignment.Center);
            listView4.Columns[2].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right2", 45, HorizontalAlignment.Center);
            listView4.Columns[3].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Turn ID", 50, HorizontalAlignment.Center);
            listView4.Columns[4].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Speed Lmt", 65, HorizontalAlignment.Center);
            listView4.Columns[5].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Trk Lgth", 60, HorizontalAlignment.Center);
            listView4.Columns[6].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Turn St", 50, HorizontalAlignment.Center);
            listView4.Columns[7].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Left Sens", 60, HorizontalAlignment.Center);
            listView4.Columns[8].TextAlign = HorizontalAlignment.Center;
            listView4.Columns.Add("Right Sens", 65, HorizontalAlignment.Center);
            listView4.Columns[9].TextAlign = HorizontalAlignment.Center;

            listView5.GridLines = true;
            listView5.View = View.Details;
            listView5.Sorting = SortOrder.None;
            listView5.Columns.Add("Train ID", 60, HorizontalAlignment.Center);       //Setup listview
            listView5.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView5.Columns.Add("Turn ID", 60, HorizontalAlignment.Center);
            listView5.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView5.Columns.Add("Turn St", 60, HorizontalAlignment.Center);
            listView5.Columns[2].TextAlign = HorizontalAlignment.Center;

            listView6.GridLines = true;
            listView6.View = View.Details;
            listView6.Sorting = SortOrder.None;
            listView6.Columns.Add("Train ID", 55, HorizontalAlignment.Center);           //Setup listview
            listView6.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView6.Columns.Add("Track ID", 60, HorizontalAlignment.Center);
            listView6.Columns[1].TextAlign = HorizontalAlignment.Center;

            listView7.GridLines = true;
            listView7.View = View.Details;
            listView7.Sorting = SortOrder.None;
            listView7.Columns.Add("Train ID", 60, HorizontalAlignment.Center);           //Setup listview
            listView7.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Owner", 45, HorizontalAlignment.Center);
            listView7.Columns[1].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Track ID", 60, HorizontalAlignment.Center);
            listView7.Columns[2].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Request", 60, HorizontalAlignment.Center);
            listView7.Columns[3].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Blocked", 60, HorizontalAlignment.Center);
            listView7.Columns[4].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Speed", 45, HorizontalAlignment.Center);
            listView7.Columns[5].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Direct.", 60, HorizontalAlignment.Center);
            listView7.Columns[6].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Orient.", 60, HorizontalAlignment.Center);
            listView7.Columns[7].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Light", 45, HorizontalAlignment.Center);
            listView7.Columns[8].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Horn", 40, HorizontalAlignment.Center);
            listView7.Columns[9].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Next Sens", 65, HorizontalAlignment.Center);
            listView7.Columns[10].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Speed Lmt", 65, HorizontalAlignment.Center);
            listView7.Columns[11].TextAlign = HorizontalAlignment.Center;
            listView7.Columns.Add("Color", 40, HorizontalAlignment.Center);
            listView7.Columns[11].TextAlign = HorizontalAlignment.Center;

            listView9.GridLines = true;
            listView9.View = View.Details;
            listView9.Sorting = SortOrder.None;
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
            listView9.Columns.Add(" ", 15, HorizontalAlignment.Center);
        }

        public void SetupColors()
        {
            StreamReader sr;

            if (!File.Exists("colors.txt"))
            //Open up default track layout for the server to get the track parts and default settings a little more dynamically
            {
                MessageBox.Show("Color file does not exist - starting standard");

                comboBox3.Items.Add("blue");
                comboBox3.Items.Add("red");

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default colors set from hard-coded");
            }
            else
            {
                sr = new StreamReader("colors.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    comboBox3.Items.Add(line);
                }

                sr.Close();

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default colors set from file");
            }

            comboBox3.SelectedIndex = 0;
        }

        public void SetupTrackParts()
        {
            StreamReader sr;
            ListViewItem lvi;
            ListViewItem.ListViewSubItem lviSub1;
            int counter = 0;

            if (!File.Exists("track.txt"))
            //Open up default track layout for the server to get the track parts and default settings a little more dynamically
            {
                MessageBox.Show("Track file does not exist - Closing program as this is crucial.  Review help to set this up");

                Form1_FormClosing(null, null);
            }
            else
            {
                sr = new StreamReader("track.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    string[] dd = line.Split(';');

                    lvi = new ListViewItem(dd[0].ToString());                              // Add left pointer

                    for (int i = 1; i < 10; i++)
                    {
                        lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[i].ToString());     // Add left pointer2
                        lvi.SubItems.Add(lviSub1);
                    }

                    listView4.Items.Add(lvi);

                    lvi = new ListViewItem("none");

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[4].ToString());     // Add turnout id
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[7].ToString());     // Add turnout state
                    lvi.SubItems.Add(lviSub1);

                    listView5.Items.Add(lvi);

                    listBox2.Items.Add(counter.ToString() + " - Available");

                    counter++;
                }

                listBox2.SelectedIndex = 0;              //Select the first index
                sr.Close();                              //Close file reader

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default track set from file");
            }
        }

        public void SetupTurnouts()
        {
            StreamReader sr;
            ListViewItem lvi;
            ListViewItem.ListViewSubItem lviSub1;

            if (!File.Exists("turnouts.txt"))
            //Open up default turnout layout for the server to get the turnout names and default settings a little more dynamically
            {
                MessageBox.Show("Turnouts file does not exist - starting standard");
                int offset = 4;

                for (int i = 0; i < 6; i++)
                {
                    lvi = new ListViewItem(offset.ToString());

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, "top");     // Add to listview
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, "none");    // Add to listview
                    lvi.SubItems.Add(lviSub1);

                    listView2.Items.Add(lvi);
                    offset = (offset * 2);
                }

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default turnouts set from program hard-coded");
            }
            else
            {
                sr = new StreamReader("turnouts.txt");
                String line;

                while ((line = sr.ReadLine()) != null)
                {
                    string[] dd = line.Split(';');

                    lvi = new ListViewItem(dd[0].ToString());

                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[1].ToString());     // Add to listview
                    lvi.SubItems.Add(lviSub1);
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, dd[2].ToString());     // Add to listview
                    lvi.SubItems.Add(lviSub1);

                    listView2.Items.Add(lvi);
                }

                sr.Close();                              //Close file reader

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Default turnouts set from file");
            }

            byte direction = 1;

            for (int i = 0; i < listView2.Items.Count; i++)
            {
                if (listView2.Items[i].SubItems[1].Text == "bottom")
                {
                    direction = 0;
                }
                else
                {
                    direction = 1;
                }

                Thread.Sleep(500);                                    // Needed otherwise command doesn't always go through

                SwitchTurnout(Convert.ToUInt32(listView2.Items[i].SubItems[0].Text), direction);     // Initially setup turnouts to structure
            }
        }

        public void SetupBeginningLabels(Label label, int i)
        {
            label.BackColor = Color.FromArgb(117, 131, 58);
            label.ForeColor = Color.FromArgb(243, 237, 156);
            label.Text = listView2.Items[i].SubItems[0].Text;
        }

        public void SetupLaterLabels(Label label)
        {
            label.ForeColor = Color.FromArgb(180, 210, 40);
            label.BackColor = Color.FromArgb(117, 131, 58);
        }

        public void SetLabelAttributes()
        {
            //Set up label colors and attributes for turnouts - (labels 6-11)
            SetupBeginningLabels(label6, 0);
            SetupBeginningLabels(label7, 1);
            SetupBeginningLabels(label8, 2);
            SetupBeginningLabels(label9, 3);
            SetupBeginningLabels(label11, 4);
            SetupBeginningLabels(label10, 5);

            //Set up the combox attributes
            comboBox1.SelectedIndex = 0;
            comboBox2.SelectedIndex = 0;

            //Set up label colors for track pieces - (labels 23-53)
            SetupLaterLabels(label23);
            SetupLaterLabels(label24);
            SetupLaterLabels(label25);
            SetupLaterLabels(label26);
            SetupLaterLabels(label27);
            SetupLaterLabels(label28);
            SetupLaterLabels(label29);
            SetupLaterLabels(label30);
            SetupLaterLabels(label31);
            SetupLaterLabels(label32);
            SetupLaterLabels(label33);
            SetupLaterLabels(label34);
            SetupLaterLabels(label35);
            SetupLaterLabels(label36);
            SetupLaterLabels(label37);
            SetupLaterLabels(label38);
            SetupLaterLabels(label39);
            SetupLaterLabels(label40);
            SetupLaterLabels(label41);
            SetupLaterLabels(label42);
            SetupLaterLabels(label43);
            SetupLaterLabels(label44);
            SetupLaterLabels(label45);
            SetupLaterLabels(label46);
            SetupLaterLabels(label47);
            SetupLaterLabels(label48);
            SetupLaterLabels(label49);
            SetupLaterLabels(label50);
            SetupLaterLabels(label51);
            SetupLaterLabels(label52);
            SetupLaterLabels(label53);
        }

        public void DefaultImages()
        {
            //Top sensors
            sensors[0] = pictureBox15;
            sensors[1] = pictureBox16;
            sensors[2] = pictureBox17;
            sensors[3] = pictureBox18;
            sensors[4] = pictureBox19;
            sensors[5] = pictureBox20;
            sensors[6] = pictureBox21;
            sensors[7] = pictureBox22;
            sensors[8] = pictureBox23;
            sensors[9] = pictureBox24;
            sensors[10] = pictureBox25;
            sensors[11] = pictureBox26;
            sensors[12] = pictureBox27;
            sensors[13] = pictureBox28;
            sensors[14] = pictureBox29;

            //Curve sensors
            sensors[15] = pictureBox30;
            sensors[16] = pictureBox31;
            sensors[17] = pictureBox32;
            sensors[18] = pictureBox33;
            sensors[19] = pictureBox34;
            sensors[20] = pictureBox35;

            //Bottom sensors
            sensors[21] = pictureBox36;
            sensors[22] = pictureBox37;
            sensors[23] = pictureBox38;
            sensors[24] = pictureBox39;
            sensors[25] = pictureBox40;
            sensors[26] = pictureBox41;
            sensors[27] = pictureBox42;
            sensors[28] = pictureBox43;

            for (int i = 0; i < sensor_count; i++)
            {
                sensors[i].Visible = false;
            }

            //Top-level track
            track[0] = pictureBox44;
            track[1] = pictureBox46;
            track[2] = pictureBox47;
            track[3] = pictureBox48;
            track[4] = pictureBox49;
            track[5] = pictureBox50;
            track[6] = pictureBox51;

            //Lower-level track
            track[7] = pictureBox47;
            track[8] = pictureBox52;
            track[9] = pictureBox53;
            track[10] = pictureBox54;
            track[11] = pictureBox55;
            track[12] = pictureBox57;
            track[13] = pictureBox54;
            track[14] = pictureBox58;

            //Bottom-level track
            track[15] = pictureBox59;
            track[16] = pictureBox60;
            track[17] = pictureBox61;
            track[18] = pictureBox62;
            track[19] = pictureBox63;
            track[20] = pictureBox65;
            track[21] = pictureBox66;
            track[22] = pictureBox67;

            //Left curve track
            track[23] = pictureBox45;
            track[24] = pictureBox68;
            track[25] = pictureBox69;
            track[26] = pictureBox62;

            //Right curve track
            track[27] = pictureBox63;
            track[28] = pictureBox71;
            track[29] = pictureBox70;
            track[30] = pictureBox56;

            for (int j = 0; j < track_count; j++)
            {
                track[j].Visible = false;
            }

            //Show default turnout images
            ShowTurnoutImages(0, pictureBox3, pictureBox4);
            ShowTurnoutImages(1, pictureBox5, pictureBox6);
            ShowTurnoutImages(2, pictureBox7, pictureBox8);
            ShowTurnoutImages(3, pictureBox9, pictureBox10);
            ShowTurnoutImages(4, pictureBox13, pictureBox14);
            ShowTurnoutImages(5, pictureBox11, pictureBox12);
        }

        public void ShowTurnoutImages(int turnout_id, PictureBox bottom, PictureBox top)
        {
            if (listView2.Items[turnout_id].SubItems[1].Text == "top")
            {
                top.Image = Properties.Resources.red_turnout;
                bottom.Image = Properties.Resources.off_turnout;
            }
            else
            {
                top.Image = Properties.Resources.off_turnout;
                bottom.Image = Properties.Resources.green_turnout;
            }
        }

        public void WritetoLogs(string loginfo)
        {
            listBox1.Items.Add(loginfo);
        }

        public void UpdateLabels(Label label, string item)
        {
            label.Text = item;
        }

        public void HandleClientConnect(IAsyncResult sync_result)
        // ################################################################################################################
        // Connection to client initiated
        //   - New connection attempt has been made, process
        // ################################################################################################################
        {
            //See if there is an open train, if there is, give this to the connection


            //Else, send back closing response to client
            /*string server_response = "NO_TRAINS";
                    byte[] response = Encoding.ASCII.GetBytes(server_response);
                    TempSocket.BeginSend(response, 0, response.Length, SocketFlags.None, new AsyncCallback(ServerSendResponse), TempSocket);*/
            try
            {
                Clients[total_connections] = (Socket)ServerSocket.EndAccept(sync_result);               // Process EndAccept and save socket information

                ListViewItem lvi;
                ListViewItem.ListViewSubItem lviSub1;                                                   // Create listview items

                lvi = new ListViewItem("");
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, total_connections.ToString());     // Add to listview
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, Clients[total_connections].RemoteEndPoint.ToString());     // Add to listview
                lvi.SubItems.Add(lviSub1);

                listView1.Items.Add(lvi);

                listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server connected to " + Clients[total_connections].RemoteEndPoint.ToString());

                HandleDatafromClient(Clients[total_connections]);                                       // Call HandleDatafromClient to set up Callback

                total_connections++;                                                                    // Increase connections
                connection_count++;

                textBox1.Enabled = true;
                button1.Enabled = true;

                label2.Invoke(new UpdateLabelsCallback(UpdateLabels), label2, "Connections: " + connection_count);

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                 // Start another callback to listen for more connections
            }
            catch (ObjectDisposedException)
            {
                //Messagebox.Show("No connection was made - exception thrown here");
            }
            catch (SocketException ex)
            {
                //listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue2 " + ex);         // If something with socket fails
            }
        }

        public void HandleDatafromClient(Socket HolderSocket)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
        {
            try
            {
                if (CallBack == null)
                {
                    CallBack = new AsyncCallback(HandleDataReceivedfromClient);                         // Set it up to create a callback to HandleDataReceivedfromClient
                }

                HolderSocket.BeginReceive(dataholder, 0, dataholder.Length, SocketFlags.None, CallBack, HolderSocket); // BeginReceive on socket
            }
            catch (SocketException ex)
            {
                //listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue3 " + ex);           // If something with socket fails
            }
        }

        public void HandleDataReceivedfromClient(IAsyncResult sync_result)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
        {
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;                    // Get information sent in temporary socket

                int received = TempSocket.EndReceive(sync_result);              // Store it in an int to then decode

                if (received > 0)
                {
                    string received_info = Encoding.ASCII.GetString(dataholder, 0, received);    // Convert received to string

                    listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - From Client " + TempSocket.RemoteEndPoint.ToString() + " " + received_info);    //Output string received


                    // ****************************************************************************************************
                    //
                    //
                    //           PROCESS REQUEST FROM CLIENT AND SEND A RESPONSE OF SOMETHING TO THEM
                    //
                    //
                    // ****************************************************************************************************



                    if (received_info == "Increase a")
                    {
                        a++;
                    }
                    else
                    {
                        a--;
                    }

                    string server_response = "Server Response";
                    byte[] response = Encoding.ASCII.GetBytes(server_response);
                    TempSocket.BeginSend(response, 0, response.Length, SocketFlags.None, new AsyncCallback(ServerSendResponse), TempSocket);

                    listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent response of " + server_response + " to Client " + TempSocket.RemoteEndPoint.ToString());
                }
                else
                {
                    CloseSocket(TempSocket);
                }
            }
            catch (ObjectDisposedException)
            {
                CloseSocket(TempSocket);
            }
            catch (SocketException)
            {
                CloseSocket(TempSocket);
            }
            catch (Exception)
            {
                CloseSocket(TempSocket);
            }
        }

        public void ServerSendResponse(IAsyncResult sync_result)
        {
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;

                int received = TempSocket.EndReceive(sync_result);
            }
            catch (SocketException ex)
            {
                //listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue4 " + ex);           // If something with socket fails
                CloseSocket(TempSocket);
            }

            HandleDatafromClient(TempSocket);
        }


        private void CloseSocket(Socket TempSocket)
        {
            for (int i = 0; i < connection_count; i++)
            {
                if (TempSocket.RemoteEndPoint.ToString() == listView1.Items[i].SubItems[2].Text)
                {
                    listView1.Items[i].Remove();

                    listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Client " + TempSocket.RemoteEndPoint.ToString() + " disconnected");

                    TempSocket.Close();
                    TempSocket = null;

                    connection_count--;

                    if (connection_count == 0)
                    {
                        textBox1.Enabled = false;
                        button1.Enabled = false;
                    }

                    label2.Invoke(new UpdateLabelsCallback(UpdateLabels), label2, "Connections: " + connection_count);

                    return;
                }
            }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (ServerSocket != null)
                {
                    ServerSocket.Close();
                }
                for (int i = 0; i < total_connections; i++)     //Go through connection count, close client sockets
                {
                    Clients[i].Close();                     //Close and null it
                }
            }
            catch (Exception exc)
            {
                //MessageBox.Show("Issue closing Server Socket and clients");
            }

            try
            {
                port.Close();
            }
            catch
            {
                //MessageBox.Show("Problem Closing Thread test.exe " + exc.ToString());
            }

            Application.Exit();
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            try
            {
                byte[] data = Encoding.ASCII.GetBytes(textBox1.Text);

                for (int i = 0; i < connection_count; i++)
                {
                    try
                    {
                        Clients[Convert.ToInt32(listView1.Items[i].SubItems[1].Text)].Send(data);
                    }
                    catch             //Client is no longer there, move up other clients
                    {
                        MessageBox.Show("Client " + listView1.Items[i].SubItems[1].Text + " disconnected?");
                    }
                }
            }
            catch (SocketException ex)
            {
                //listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue5 " + ex);           // If something with socket fails
            }

            listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent " + textBox1.Text + " to Clients");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //label4.Invoke(new UpdateLabelsCallback(UpdateLabels), label4, a.ToString());                                         // Update this
            button5_Click(sender, e);
        }

        //######################################################################
        //        TURNOUT EVENTS
        //######################################################################

        private void pictureBox4_Click(object sender, EventArgs e)
        {
            CheckTurnout(0, "top", pictureBox3, pictureBox4);
        }
        private void pictureBox3_Click(object sender, EventArgs e)
        {
            CheckTurnout(0, "bottom", pictureBox3, pictureBox4);
        }


        private void pictureBox6_Click(object sender, EventArgs e)
        {
            CheckTurnout(1, "top", pictureBox5, pictureBox6);
        }
        private void pictureBox5_Click(object sender, EventArgs e)
        {
            CheckTurnout(1, "bottom", pictureBox5, pictureBox6);
        }


        private void pictureBox8_Click(object sender, EventArgs e)
        {
            CheckTurnout(2, "top", pictureBox7, pictureBox8);
        }
        private void pictureBox7_Click(object sender, EventArgs e)
        {
            CheckTurnout(2, "bottom", pictureBox7, pictureBox8);
        }


        private void pictureBox10_Click(object sender, EventArgs e)
        {
            CheckTurnout(3, "top", pictureBox9, pictureBox10);
        }
        private void pictureBox9_Click(object sender, EventArgs e)
        {
            CheckTurnout(3, "bottom", pictureBox9, pictureBox10);
        }


        private void pictureBox14_Click(object sender, EventArgs e)
        {
            CheckTurnout(4, "top", pictureBox13, pictureBox14);
        }
        private void pictureBox13_Click(object sender, EventArgs e)
        {
            CheckTurnout(4, "bottom", pictureBox13, pictureBox14);
        }


        private void pictureBox12_Click(object sender, EventArgs e)
        {
            CheckTurnout(5, "top", pictureBox11, pictureBox12);
        }
        private void pictureBox11_Click(object sender, EventArgs e)
        {
            CheckTurnout(5, "bottom", pictureBox11, pictureBox12);
        }


        public void CheckTurnout(int turnout_id, string location, PictureBox bottom, PictureBox top)
        {
            //Enter monitor
            Monitor.Enter(listView2);

            byte direction = 1;

            if (listView2.Items[turnout_id].SubItems[2].Text == "none" && listView2.Items[turnout_id].SubItems[1].Text != location)
            {
                listView2.Items[turnout_id].SubItems[1].Text = location;

                if (location == "bottom")
                {
                    direction = 0;
                }

                SwitchTurnout(Convert.ToUInt32(listView2.Items[turnout_id].SubItems[0].Text), direction);

                ShowTurnoutImages(turnout_id, bottom, top);

                listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Turnout " + listView2.Items[turnout_id].SubItems[0].Text + " changed to " + listView2.Items[turnout_id].SubItems[1].Text);
            }

            Monitor.Exit(listView2);
            //Exit monitor
        }

        public void SwitchTurnout(uint turnout, byte direction)
        // ##################################################################
        // Taken from DCCMan.cpp and converted to C#
        //    > Unsigned chars are now bytes
        // ##################################################################
        {
            byte[] command = new byte[6];

            /*byte b1 = (byte)(0x80 + ((turnout-1)>>2) +1);				// Address (shifts two to the right)
	        byte newadd = (byte)((turnout-1) & 0x03);			        // Bit mask to get first to address bits
	        byte b2 = (byte)(0xF0 + 8 + (newadd<<1) + direction);		// Address and state
	        byte b3 = (byte)(b1 ^ b2);							        // Checksum

	        command[0] = 0x93;	                                        // NCE command (3 byte packet)
	        command[1] = 0x0A;	                                        // Repeat ten times
	        command[2] = b1;	                                        // First byte of packet
	        command[3] = b2;
	        command[4] = b3;*/

            command[0] = 0xAD;                                          // Standard accessory packet command
            command[1] = 0;                                             // High address, here set to 0 to be simple
            command[2] = (byte)turnout;                                 // Address of turnout
            command[3] = (byte)(04 - direction);                        // Direction
            command[4] = 0;                                             // Default data

            port.Write(command, 0, 5);
        }

        private void pictureBox72_Click(object sender, EventArgs e)
        {
            string[] data = new string[10];
            bool bad_train = false;
            int train_id;

            //Add Train

            //Verify that train is in an int
            try
            {
                train_id = Convert.ToInt32(textBox2.Text);

                //Verify that train id is within 1-500
                if (train_id > 0 && train_id < 501)
                {
                    //Verify that train id is not in use
                    for (int i = 0; i < listView1.Items.Count; i++)
                    {
                        if (listView1.Items[i].SubItems[0].Text == train_id.ToString())
                        {
                            bad_train = true;
                        }
                    }
                }
                else
                {
                    bad_train = true;
                }
            }
            catch
            {
                MessageBox.Show("Train is already in use or Short ID not within 1-500");
                return;
            }

            if (bad_train == true)
            {
                MessageBox.Show("Train is already in use or Short ID not within 1-500");
                return;
            }

            //Verify that colors are valid and one is chosen
            try
            {
                if (comboBox3.Text.Length == 0)
                {
                    MessageBox.Show("No more trains can be added at this time");
                    return;
                }
            }
            catch
            {
                MessageBox.Show("No more trains can be added at this time");
                return;
            }

            data[0] = train_id.ToString();                // Train ID
            data[1] = comboBox1.Text;                     // Direction
            data[2] = comboBox2.Text;                     // Facing
            data[3] = comboBox3.Text;                     // Color
            data[4] = listBox2.SelectedIndex.ToString();

            CommandRailroad("ADD_TRAIN", data);
        }

        public void CommandRailroad(string command, string[] data)
        {
            //Enter monitors
            Monitor.Enter(listView5);
            Monitor.Enter(listView2);
            Monitor.Enter(listView7);

            if (command == "ADD_TRAIN")
            {
                MessageBox.Show("Train ID: " + data[0] + "   -   Direction: " + data[1] + "   -   Facing: " + data[2] + "   -   Color: " + data[3] + "   -   SelectedTrack: " + data[4]);

                //Check if there is a turnout associated with resource
                if (listView5.Items[Convert.ToInt32(data[4])].SubItems[1].Text == "none")
                {
                    //No turnout for track, check if resource is open
                    if (listView5.Items[Convert.ToInt32(data[4])].SubItems[0].Text == "none")
                    {
                        //No trains associated with track
                    }
                    else
                    {
                        Monitor.Exit(listView7);
                        Monitor.Exit(listView2);
                        Monitor.Exit(listView5);
                        MessageBox.Show("Track is in use - Please select another");
                        return;
                    }
                }
                else
                {
                    //Verify that track is unused and turnout is turned in right direction
                    if (listView5.Items[Convert.ToInt32(data[4])].SubItems[0].Text == "none" &&
                        listView2.Items[Convert.ToInt32(listView5.Items[Convert.ToInt32(data[4])].SubItems[1].Text)].SubItems[1].Text == listView5.Items[Convert.ToInt32(data[4])].SubItems[2].Text)
                    {
                        //No trains associated with track and turnout
                        listView2.Items[Convert.ToInt32(listView5.Items[Convert.ToInt32(data[4])].SubItems[1].Text)].SubItems[2].Text = data[0];     // Set TrainID for this Turnout ownership
                    }
                    else
                    {
                        Monitor.Exit(listView7);
                        Monitor.Exit(listView2);
                        Monitor.Exit(listView5);
                        MessageBox.Show("Track is in use - Please select another");
                        return;
                    }
                }

                //Add Train to Trains and connections
                ListViewItem lvi;
                ListViewItem.ListViewSubItem lviSub1;                                                   // Create listview items

                lvi = new ListViewItem(data[0]);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "");     // Add to listview
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "Available");     // Add to listview
                lvi.SubItems.Add(lviSub1);

                listView1.Items.Add(lvi);

                //Add Train Information
                lvi = new ListViewItem(data[0]);                             // Train ID
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "none");     // Ownership
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "none");     // Track ID
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, data[4]);    // Request
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "false");    // Blocked
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "0");        // Speed
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, data[1]);    // Direction
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, data[2]);    // Orientation
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "false");    // Light
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, "false");    // Horn
                lvi.SubItems.Add(lviSub1);

                //Look at direction and orientation to determine next sensor
                if ((data[2] == "Left" && data[1] == "Forward") || (data[2] == "Right" && data[1] == "Reverse"))
                {
                    //Train should be going left, pick the right sensor for now
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, listView4.Items[Convert.ToInt32(data[4])].SubItems[9].Text);      // Next Sensor
                    lvi.SubItems.Add(lviSub1);
                }
                else
                {
                    //Train should be going right, pick the left sensor for now
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, listView4.Items[Convert.ToInt32(data[4])].SubItems[8].Text);      // Next Sensor
                    lvi.SubItems.Add(lviSub1);
                }

                lviSub1 = new ListViewItem.ListViewSubItem(lvi, listView4.Items[Convert.ToInt32(data[4])].SubItems[5].Text);     // Speed Limit
                lvi.SubItems.Add(lviSub1);
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, data[3]);    // Color
                lvi.SubItems.Add(lviSub1);

                listView7.Items.Add(lvi);

                //Remove color taken
                comboBox3.Items.Remove(comboBox3.SelectedItem);
                try
                {
                    comboBox3.SelectedIndex = 0;
                }
                catch { }

                //Add train to delete train list
                listBox3.Items.Add("Train " + data[0]);

                listView5.Items[Convert.ToInt32(data[4])].SubItems[0].Text = data[0];    // Set TrainID for Resource ownership
                listBox2.Items[Convert.ToInt32(data[4])] = data[4] + " - Used";
                total_trains++;

                //Exit monitors
                Monitor.Exit(listView7);
                Monitor.Exit(listView2);
                Monitor.Exit(listView5);

                listBox1.Invoke(new WritetoLogsCallback(WritetoLogs), DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Train " + data[0] + " added to railroad");                //Put in a sensor hit on the sensor away from where it is going
                SensorHitChange(data[0]);
                return;
            }

            //Exit monitors
            Monitor.Exit(listView7);
            Monitor.Exit(listView2);
            Monitor.Exit(listView5);
        }

        public void SensorHitChange(string train_id)
        {
            //Enter monitors

            int train_location = GetTrainLocation(train_id);

            //View requested track of this train and make it it's new owned track
            listView7.Items[train_location].SubItems[2].Text = listView7.Items[train_location].SubItems[3].Text;
            listView7.Items[train_location].SubItems[3].Text = "none";

            //Change image on tracked_id associated with this train with the proper color
            string imagename = listView7.Items[train_location].SubItems[2].Text + "_" + listView7.Items[train_location].SubItems[12].Text;
            track[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)].Image = (Image)Properties.Resources.ResourceManager.GetObject(imagename);
            track[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)].Visible = true;

            //Exit monitors



            RequestNextTrack(train_id);
        }

        private void pictureBox72_MouseHover(object sender, EventArgs e)
        {
            pictureBox72.Image = Properties.Resources.add_train_over;
        }

        private void pictureBox76_MouseHover(object sender, EventArgs e)
        {
            pictureBox76.Image = Properties.Resources.remove_over;
        }

        private void pictureBox72_MouseLeave(object sender, EventArgs e)
        {
            pictureBox72.Image = Properties.Resources.add_train;
        }

        private void pictureBox76_MouseLeave(object sender, EventArgs e)
        {
            pictureBox76.Image = Properties.Resources.remove;
        }

        public int GetTrainLocation(string train_id)
        {
            int train_location = 0;

            //Find the train item associated with this train_id
            for (int i = 0; i < listView7.Items.Count; i++)
            {
                if (listView7.Items[i].SubItems[0].Text == train_id)
                {
                    train_location = i;
                }
            }

            return train_location;
        }

        public void RequestNextTrack(string train_id)
        {
            //Enter monitors


            int train_location = GetTrainLocation(train_id);

            //Get the next requested track and see if it is available and turnout is in the right direction if applicable
            if ((listView7.Items[train_location].SubItems[7].Text == "Left" && listView7.Items[train_location].SubItems[6].Text == "Forward") ||
                (listView7.Items[train_location].SubItems[7].Text == "Right" && listView7.Items[train_location].SubItems[6].Text == "Reverse"))
            {
                //Train should be going left, pick the next track and sensor
                MessageBox.Show("Train should be going left");

                if (listView4.Items[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)].SubItems[1].Text == "none")
                {
                    //Only one track piece that can be selected for this direction

                    //See if turnout is associated with this piece and if it's turned the right way
                    /*if (listView4.Items[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)].SubItems[4].Text != "none")
                    {
                        //Turnout associated with this area, check if this track is owned and it's turnout state is in the right direction
                        if(listView5.Items[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)]
                            //listView4.Items[Convert.ToInt32(listView7.Items[train_location].SubItems[2].Text)].SubItems[4].Text 
                    }*/

                }
                else
                {
                    //Look at which way the turnout is pointed to then see what track is requested


                    //See if the turnout is owned by anyone else


                }
            }
            else
            {
                //Train should be going right, pick the next track and sensor
                MessageBox.Show("Train should be going right");

                if (listView7.Items[train_location].SubItems[2].Text != "none")
                {
                    //Only track piece that can be selected for this direction


                    //See if turnout is associated with this piece and if it's turned the right way


                }
                else
                {
                    //Look at which way the turnout is pointed to then see what track is requested


                    //See if the turnout is owned by anyone else

                }
            }



            //If it is not, add it to the blocked listview with this as the track_id
            //If it is, assign this as the Requested and change ownership on this piece - change turnout ownership if needed also

            //Get next sensor information, etc.


            //Exit monitors
        }

        public int speed = 0;
        public byte dir = 1;

        private void button4_Click(object sender, EventArgs e)
        {
            //Move train forward
            //byte[] command = new byte[4];
            byte[] command = new byte[6];

            /*command[0] = 0xA3; // opcode for 3 byte packet
	        command[1] = (byte)0002;

            speed = speed + 2;

            dir = (byte)(dir<<5); // shift left 5
		    command[2] = (byte)(0x40 | dir | (byte)speed);  */

            command[0] = 0xA2;
            command[1] = 0x00;
            command[2] = 0x03;
            command[3] = 02;
            command[4] = 120;

            port.Write(command, 0, 5);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            //Stop train
            byte[] command = new byte[6];

            command[0] = 0xA2;
            command[1] = 0;
            command[2] = 3;
            command[3] = 02;
            command[4] = 0;

            port.Write(command, 0, 5);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            //Move train backward
            byte[] command = new byte[6];

            command[0] = 0xA2;
            command[1] = 0x00;
            command[2] = 0x03;
            command[3] = 01;
            command[4] = 120;

            port.Write(command, 0, 5);
        }

        private void button5_Click(object sender, EventArgs e)
        {
            // Messagebox of the current AIU status
            byte[] command = new byte[3];
            int[] value = new int[15];

            listView9.Items.Clear();

            for (int i = 50; i <= 62; i=i+4)
            {
                command[0] = 0x9B;
                command[1] = (byte)i;
                
                port.Write(command, 0, 2);

                char[] buffer = new char[8];

                for (int j = 0; j < 8; j++)
                {
                    buffer[j] = ' ';
                }

                port.Read(buffer, 0, 2);

	            for(int j=0;j<6;j++)
	            {
		            char test = buffer[0];
		            test = (char)(test >> j);
		            value[j+8] = (test & 0x01);
	            }

	            for(int j=0;j<8;j++)
	            {
		            char test = buffer[1];
		            test = (char)(test >> j);
		            value[j] = (test & 0x01);
	            }

                ListViewItem lvi;
                ListViewItem.ListViewSubItem lviSub1;

                lvi = new ListViewItem(value[0].ToString());

	            for(int j=1; j<14; j++)
	            {
                    lviSub1 = new ListViewItem.ListViewSubItem(lvi, value[j].ToString());     // Add to listview
                    lvi.SubItems.Add(lviSub1);
	            }

                listView9.Items.Add(lvi);

                //MessageBox.Show("Sensor " + i.ToString() + ": " + buffer[0].ToString() + " - " + buffer[1].ToString());
            }
        }
    }
}